package com.ywt.mg.services;

import com.ywt.mg.core.SqlHelper;
import com.ywt.mg.core.utils.Checker;
import com.ywt.mg.core.utils.FormatUtil;
import com.ywt.mg.core.utils.IdCardUtil;
import com.ywt.mg.domain.entities.NatOrder;
import com.ywt.mg.domain.entities.OrderPayment;
import com.ywt.mg.domain.models.ConstantDef;
import com.ywt.mg.domain.models.ExcelDataMap;
import com.ywt.mg.domain.models.enums.DatePeriodEnum;
import com.ywt.mg.domain.models.enums.NatOrderStatusEnum;
import com.ywt.mg.domain.models.enums.PaymentStatusEnum;
import com.ywt.mg.domain.models.enums.RefundStatusEnum;
import com.ywt.mg.domain.models.pojo.ExcelCollectPojo;
import com.ywt.mg.params.deposit.QueryDepositListRequest;
import com.ywt.mg.params.natOrder.NatOrderListRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class NatOrderService {

    private static Logger logger = LoggerFactory.getLogger(NatOrderService.class);

    @Autowired
    private SqlHelper sqlHelper;

    @Autowired
    private AuthService authService;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private OrderPaymentService orderPaymentService;

    @Autowired
    private HospitalCacheService hospitalCacheService;

    @Autowired
    private DownloadRecordService downloadRecordService;


    public List<NatOrder> queryNatOrderList(NatOrderListRequest request) throws Exception {
        Map<String, Object> map = queryNatOrderListCommon(request);
        String whereSql = (String) map.get(ConstantDef.KEY_WHERE_SQL);
        List<Object> paramList = (List<Object>) map.get(ConstantDef.KEY_PARAM_LIST);
        return sqlHelper.getListWithNoPage("*", whereSql, " id desc ", NatOrder.class, paramList.toArray());
    }

    private Map<String, Object> queryNatOrderListCommon(NatOrderListRequest request) throws Exception {
        List<Object> paramList = new ArrayList<>();
        // 只显示未删除和支付成功的记录
        String whereSql = " ( 1=1 and deleted = 0 and payment_status = 2 )";

        String paymentCode = request.getPaymentCode().trim();
        if (!Checker.isNull(paymentCode)) {
            whereSql += " and ( payment_code = ?)";
            paramList.add(paymentCode);
        }

        String patientName = request.getPatientName().trim();
        if (!Checker.isNull(patientName)) {
            whereSql += " and ( patient_name like ?)";
            paramList.add(patientName + "%");
        }

        String idCard = request.getIdCard().trim();
        if (!Checker.isNull(idCard)) {
            whereSql += " and ( id_card = ?)";
            paramList.add(idCard);
        }

        String mobile = request.getMobile().trim();
        if (!Checker.isNull(mobile)) {
            whereSql += " and ( mobile like ?)";
            paramList.add(mobile + "%");
        }

        String orderNo = request.getOrderNo().trim();
        if (!Checker.isNull(orderNo)) {
            whereSql += " and ( order_no like ?)";
            paramList.add(orderNo + "%");
        }

        String vaccineCode = request.getVaccineCode().trim();
        if (!Checker.isNull(vaccineCode)) {
            whereSql += " and ( vaccine_code like ?)";
            paramList.add(vaccineCode + "%");
        }

        String paymentStatus = request.getPaymentStatus().trim();
        if (!Checker.isNull(paymentStatus)) {
            whereSql += " and ( payment_status = ?)";
            paramList.add(Integer.parseInt(paymentStatus));
        }

        String orderStatus = request.getOrderStatus().trim();
        if (!Checker.isNull(orderStatus)) {
            int orderStatusInt = Integer.parseInt(orderStatus);
            whereSql += " and ( (status & ?) = ? and status < ? ) ";
            paramList.add(orderStatusInt);
            paramList.add(orderStatusInt);
            paramList.add(2 * orderStatusInt);
        }

        int hospitalId = request.getHospitalId();
        if (hospitalId != -1) {
            whereSql += " and ( hospital_id = ?)";
            paramList.add(hospitalId);
        }

        // 对账时间搜索
        String billEndTime = request.getBillEndTime();
        String billStartTime = request.getBillStartTime();
        if (!Checker.isNull(billEndTime)) {
            whereSql += " and ( pay_time < ? or refund_time < ?)";
            Date date = FormatUtil.stringToDateDayTimeEnd(billEndTime);
            paramList.add(date);
            paramList.add(date);
        }
        if (!Checker.isNull(billStartTime)) {
            whereSql += " and ( pay_time >= ? or refund_time >= ?)";
            Date date = FormatUtil.stringToDateDayTimeStart(billStartTime);
            paramList.add(date);
            paramList.add(date);
        }
        String transactionId = request.getTransactionId();
        try {
            // 处理交易流水号，考虑到性能，直接采用全匹配到方式
            if (!Checker.isNull(transactionId)) {
                OrderPayment orderPayment = orderPaymentService.getOrderPaymentByTransactionId(transactionId);
                if (!Checker.isNone(orderPayment)) {
                    whereSql += " and ( order_id = ? )";
                    paramList.add(orderPayment.getOrderId());
                } else {
                    whereSql += " and ( order_id = -1 )";
                }
            }
        } catch (Exception e) {
            logger.error("NatOrderService#queryNatOrderListCommon(transactionId={} ):\n {} ", transactionId, e.getMessage(), e);
        }

        String paymentStartTime = request.getPaymentStartTime().trim();
        String paymentEndTime = request.getPaymentEndTime().trim();
        Map<String, Object> map = sqlHelper.timeQuerySql(whereSql, paramList, "pay_time", paymentStartTime, paymentEndTime);

        String refundStartTime = request.getRefundStartTime().trim();
        String refundEndTime = request.getRefundEndTime().trim();
        map = sqlHelper.timeQuerySql(map, "refund_time", refundStartTime, refundEndTime);

        String confirmStartTime = request.getConfirmStartTime().trim();
        String confirmEndTime = request.getConfirmEndTime().trim();
        map = sqlHelper.timeQuerySql(map, "confirm_time", confirmStartTime, confirmEndTime, FormatUtil.FORMAT_DATE_SECOND);
        return map;
    }


    public void downloadNatOrderList(int downloadRecordId, String fileName, NatOrderListRequest request) {
        try {
            String col1 = "订单号", col1_1 = "支付流水号", col1_1_1 = "交易流水号", col2 = "医院", col3 = "患者姓名", col4 = "身份证号", col5 = "年龄", col6 = "性别", col7 = "手机号";
            String col8 = "缴费码", col9 = "疫苗码", col10 = "金额", col11 = "支付状态", col12 = "订单状态";
            String col13 = "支付时间", col14 = "退费时间", col15 = "预约时间", col16 = "执行时间", col17 = "备注";
            String[] columns = new String[]{col1, col1_1, col1_1_1, col2, col3, col4, col5, col6, col7, col8, col9, col10, col11, col12, col13, col14, col15, col16, col17};
            ExcelDataMap map = new ExcelDataMap(columns);
            List<NatOrder> list = queryNatOrderList(request);
            int payTotal = 0, refundTotal = 0, realityTotal = 0;
            List<OrderPayment> orderPaymentList = orderPaymentService.getOrderPaymentListByNatOrderList(list);
            for (NatOrder item : list) {
                // 交易流水号
                String transactionId = orderPaymentService.getTransactionIdByOrderId(orderPaymentList, Checker.getIntegerValue(item.getOrderId()));
                map.getStringListSafely(col1).add(Checker.getStringValue(item.getOrderNo()));
                map.getStringListSafely(col1_1).add(Checker.getStringValue(item.getPaymentNo()));
                map.getStringListSafely(col1_1_1).add(transactionId);
                map.getStringListSafely(col2).add(hospitalCacheService.getCacheHospitalNameByHospitalId(item.getHospitalId()));
                map.getStringListSafely(col3).add(Checker.getStringValue(item.getPatientName()));
                map.getStringListSafely(col4).add(Checker.getStringValue(item.getIdCard()));
                String idCard = Checker.getStringValue(item.getIdCard());
                if (!Checker.isNone(idCard)) {
                    int currentAge = IdCardUtil.getCurrentAge(idCard);
                    String sex = IdCardUtil.getSex(idCard);
                    map.getStringListSafely(col5).add(currentAge + "");
                    map.getStringListSafely(col6).add(sex);
                }

                map.getStringListSafely(col7).add(Checker.getStringValue(item.getMobile()));

                map.getStringListSafely(col8).add(Checker.getStringValue(item.getPaymentCode()));
                map.getStringListSafely(col9).add(Checker.getStringValue(item.getVaccineCode()));
                map.getStringListSafely(col10).add(FormatUtil.intShrink100ToStr(item.getAmount()));
                map.getStringListSafely(col11).add(PaymentStatusEnum.getPaymentStatus(item.getPaymentStatus()).getDisplayName());
                map.getStringListSafely(col12).add(NatOrderStatusEnum.valueOf(item.getStatus()).getDisplayName());

                map.getStringListSafely(col13).add(FormatUtil.createTimeFormatDetail(item.getPayTime()));
                map.getStringListSafely(col14).add(FormatUtil.createTimeFormatDetail(item.getRefundTime()));
                map.getStringListSafely(col15).add(FormatUtil.formatDate(item.getRegDate()) + " " + DatePeriodEnum.valueOf(item.getRegTimePeriod()).getDisplayName());
                map.getStringListSafely(col16).add(FormatUtil.createTimeFormatDetail(item.getConfirmTime()));
                map.getStringListSafely(col17).add(Checker.getStringValue(item.getRemark()));

                if (Checker.getIntegerValue(item.getPaymentStatus()) == PaymentStatusEnum.Success.getValue()) {
                    payTotal += Checker.getIntegerValue(item.getAmount());
                }
                if (Checker.getIntegerValue(item.getRefundStatus()) == RefundStatusEnum.SUCCESS.getValue()) {
                    refundTotal += Checker.getIntegerValue(item.getAmount());
                }
            }
            int size = list.size();
            List<ExcelCollectPojo> itemList = getStatisticsData(size, payTotal, refundTotal);
            downloadRecordService.createFileAndUploadOssAndSaveToDataBase(fileName, downloadRecordId, map, itemList);

        } catch (Exception e) {
            logger.error("/natOrder/downloadNatOrderList: {}", e.getMessage(), e);
            e.printStackTrace();
        }
    }

    private List<ExcelCollectPojo> getStatisticsData(int size, int payTotal, int refundTotal) {
        int startRows = size + 3;
        int startColumn = 3;

        List<ExcelCollectPojo> customExcelItemList = new ArrayList<>();
        customExcelItemList.add(new ExcelCollectPojo(startColumn + 1, startRows + 1, new String[]{"支付总额", FormatUtil.intShrink100ToStr(payTotal)}));
        customExcelItemList.add(new ExcelCollectPojo(startColumn + 1, startRows + 2, new String[]{"退款总额", FormatUtil.intShrink100ToStr(refundTotal)}));
        customExcelItemList.add(new ExcelCollectPojo(startColumn + 1, startRows + 3, new String[]{"实际支付金额", FormatUtil.intShrink100ToStr(payTotal - refundTotal)}));
        return customExcelItemList;
    }
}
